//***************************************************************************
// Hello USART
//
// @author Amir Lazarovich 
// @version 0.1 2013
//
// * based on hello_radio code
//**************************************************************************
#ifndef __usart_v1__
#define __usart_v1__

#include <stdio.h>

#ifndef TRUE
#define TRUE (0==0)
#endif
#ifndef FALSE
#define FALSE (0!=0)
#endif

void USART_initSerial(unsigned long fcpu, unsigned long baud);
int USART_putchar(char c, FILE *stream);
void USART_init(void);
void USART_setBaudRate(unsigned long fcpu, unsigned long baud);
void USART_transmit(unsigned char data);
int USART_isReceiveReady(void);
int USART_receive(unsigned char *data);



// *********************** inline usart.c ***********************
FILE stdout_usart[] = {FDEV_SETUP_STREAM(USART_putchar, NULL, _FDEV_SETUP_WRITE)};

void USART_init(void) { 
  /* Enable receiver and transmitter */ 
  UCSR0B = (1<<RXEN0)|(1<<TXEN0); 
  /* Set frame format: 8data, 2stop bit */ 
  UCSR0C = (1<<USBS0)|(3<<UCSZ00); 
}

void USART_setBaudRate(unsigned long fcpu, unsigned long baud) { 
  unsigned int ubrr = fcpu/8/baud - 1;

  /* Set 2X clock rate */
  UCSR0A = (1<<U2X0);
  /* Set baud rate */
  UBRR0H = (unsigned char)(ubrr>>8);
  UBRR0L = (unsigned char)ubrr;
}

void USART_transmit(unsigned char data) { 
  /* Wait for empty transmit buffer */ 
  while ( !( UCSR0A & (1<<UDRE0)) );
  /* Put data into buffer, sends the data */ 
  UDR0 = data;
}

int USART_isReceiveReady(void) { 
  /* Wait for data to be received */ 
  return (UCSR0A & (1<<RXC0)) != 0;
}

int USART_receive(unsigned char *data) { 
  if (USART_isReceiveReady()) {
    *data = UDR0;
    return TRUE;
  } else {
    return FALSE;
  }
}

/* set up stdio to USART */
int USART_putchar(char c, FILE *stream) {
  if (c == '\n')
    USART_putchar('\r', stream);
  USART_transmit(c);
  return 0;
}

void USART_initSerial(unsigned long fcpu, unsigned long baud) {
        USART_init();
        USART_setBaudRate(fcpu, baud);
        stdout = stdout_usart;
}

#endif