//***************************************************************************
// Interrupted based SPI master-slave hello world example
// Designed for ATmega328p fabduino but suits all avr mega types. 
// When in doubt, double check the pins
//
// @author Amir Lazarovich
// @version 0.1 2013
//**************************************************************************
#include "libs/avr-io.h"
#include "libs/usart.h"
#include "libs/spi.h"

#define BAUD 9600L


void onEndTransimission(void) {
        printf("Send complete\n");
}

/** 
 * Register interrupt handler for SPI 
 * note that this isn't a method declaration but only a macro call
 */
ISR(SPI_STC_vect) {
        // delegate to our SPI helper handler
        // onEndTransmission: callback that will be invoked when we finished transmitting
        SPI_handleSendInterrupt(&onEndTransimission);
}

int main (void) {
        disableClockPrescaling();
        USART_initSerial(F_CPU, BAUD); 
        SPI_initMasterIntr();

         // must come after initializing the USART serial and making sure the clock prescaler is set to 1 
        printf("\n*** MASTER mode ***\n");        

        // loop
        static unsigned char c;
        while (1) {
                if (USART_isReceiveReady()) {
                        USART_receive(&c);
                        switch (c) {
                                case 's':
                                        printf("sending on bus\n");
                                        SPI_sendIntr((char*) "this is a test");
                                        break;

                                default:
                                        printf("ignoring input (psssst! try sending 's')\n");
                        }
                }
        }
}

