//***************************************************************************
// Interrupted based SPI master-slave hello world example
// Designed for ATmega328p fabduino but suits all avr mega types. 
// When in doubt, double check the pins
//
// @author Amir Lazarovich
// @version 0.1 2013
//**************************************************************************
#include "libs/avr-io.h"
#include "libs/usart.h"
#include "libs/spi.h"

#define BAUD 9600L
#define BUFFER_SIZE 128

char buffer[BUFFER_SIZE];
char* pbuffer;


void onSPIMessageReceived(char* buffer, int status) {
    if (status == SUCCESS) {
        printf("received: %s\n", buffer);
    } else {
        printf("failure!\n");
    }
}

/** 
 * Register interrupt handler for SPI 
 * note that this isn't a method declaration but only a macro call
 */
ISR(SPI_STC_vect) {
    // delegate to our SPI helper handler
    // buffer: where we'll save the received transmission
    // buffer_size: our maxmium transmission size - if we detect a message that has a larger value we will ignore it and return a "failure" status
    // onSPIMessageReceived: callback that will be invoked when transission is over
    SPI_handleReceiveInterrupt(buffer, BUFFER_SIZE, &onSPIMessageReceived); 
}

int main (void) {
        disableClockPrescaling();
        USART_initSerial(F_CPU, BAUD); 
        SPI_initSlaveIntr();

        // must come after initializing the USART serial and making sure the clock prescaler is set to 1 
        printf("\n*** SLAVE mode ***\n");

        // loop
        while(1);
}

